@@ -1089,27 +1089,6 @@ void Envelope::GetValues(double *buffer, int bufferLen,
    double t = t0;
    double tprev, vprev, tnext = 0, vnext, vstep = 0;
 
-   // JC: I believe small rounding errors in t0 are OK,
-   // and that the previous tests (with epsilon==0)
-   // were too stringent.
-
-#if defined(__WXDEBUG__)
-   // in debug builds, do a spot of argument checking
-   if(t0 > (mTrackLen+mTrackEpsilon))
-   {
-      wxString msg;
-      msg = wxString::Format(wxT("t0 %.20f mTrackLen %.20f diff %.20f"), t0, mTrackLen, t0-mTrackLen);
-      wxASSERT_MSG(t0 <= mTrackLen, msg);
-   }
-
-   if(t0 < (-mTrackEpsilon))
-   {
-      wxString msg;
-      msg = wxString::Format(wxT("t0 %.20f"), t0);
-      wxASSERT_MSG(t0 >= 0, msg);
-   }
-#endif
-
    for (int b = 0; b < bufferLen; b++) {
 
       // Get easiest cases out the way first...
