@@ -1310,7 +1310,7 @@ bool VSTEffect::Process()
       }
 
       if (mBlockSize == 0) {
-         mBlockSize = left->GetMaxBlockSize() * 2;
+         mBlockSize = mWTBlockSize = left->GetMaxBlockSize() * 2;
 
          // Some VST effects (Antress Modern is an example), do not like
          // overly large block sizes.  Unfortunately, I have not found a
@@ -1327,9 +1327,11 @@ bool VSTEffect::Process()
             mInBuffer[i] = new float[mBlockSize];
          }
 
+         //Process 2 audacity blockfiles per WaveTrack::Set independently of mBlockSize
+         //because it is extremely slow to do multiple Set()s per blockfile.
          mOutBuffer = new float *[mOutputs];
          for (int i = 0; i < mOutputs; i++) {
-            mOutBuffer[i] = new float[mBlockSize];
+            mOutBuffer[i] = new float[mWTBlockSize + mBlockSize];
          }
 
          // Turn the power off
@@ -1385,6 +1387,7 @@ bool VSTEffect::ProcessStereo(int count,
                               sampleCount len)
 {
    bool rc = true;
+   sampleCount amountLeft = 0;
 
    // Initialize time info
    mTimeInfo.samplePos = 0.0;
@@ -1401,6 +1404,10 @@ bool VSTEffect::ProcessStereo(int count,
    sampleCount originalLen = len;
    sampleCount ls = lstart;
    sampleCount rs = rstart;
+   sampleCount outls = lstart;
+   sampleCount outrs = rstart;
+   sampleCount outBufferCursor = 0;
+   float **outBufSegment = new float *[mOutputs];
    while (len) {
       int block = mBlockSize;
       if (block > len) {
@@ -1412,12 +1419,27 @@ bool VSTEffect::ProcessStereo(int count,
          right->Get((samplePtr)mInBuffer[1], floatSample, rs, block);
       }
 
-      callProcessReplacing(mInBuffer, mOutBuffer, block);
-
-      left->Set((samplePtr)mOutBuffer[0], floatSample, ls, block);
-      if (right) {
-         right->Set((samplePtr)mOutBuffer[1], floatSample, rs, block);
-      }      
+      for (int i = 0; i < mOutputs; i++)
+         outBufSegment[i] = mOutBuffer[i ] + outBufferCursor;
+      callProcessReplacing(mInBuffer, outBufSegment, block);
+      outBufferCursor += block;
+      //Process 2 audacity blockfiles per WaveTrack::Set independently of mBlockSize
+      //because it is extremely slow to do multiple Set()s per blockfile due to Undo History
+      //If we do more optimization we should probably align the Sets to blockfile boundries.
+      if (outBufferCursor >= mWTBlockSize) {
+         left->Set((samplePtr)mOutBuffer[0], floatSample, outls, mWTBlockSize);
+         if (right) {
+            right->Set((samplePtr)mOutBuffer[1], floatSample, outrs, mWTBlockSize);
+         }
+         if (outBufferCursor >= mWTBlockSize) {
+            //snake the buffer down
+            memmove(mOutBuffer[0], mOutBuffer[0] + mWTBlockSize, SAMPLE_SIZE(floatSample) * (outBufferCursor - mWTBlockSize));
+            memmove(mOutBuffer[1], mOutBuffer[1] + mWTBlockSize, SAMPLE_SIZE(floatSample) * (outBufferCursor - mWTBlockSize));
+         }
+         outBufferCursor -= mWTBlockSize;
+         outls += mWTBlockSize;
+         outrs += mWTBlockSize;
+      }
 
       len -= block;
       ls += block;
@@ -1438,6 +1460,14 @@ bool VSTEffect::ProcessStereo(int count,
       }
    }
 
+   //finish taking the remainder.
+   if (outBufferCursor) {
+     left->Set((samplePtr)mOutBuffer[0], floatSample, outls, outBufferCursor);
+     if (right) {
+         right->Set((samplePtr)mOutBuffer[1], floatSample, outrs, outBufferCursor);
+      }
+   }
+
    // Tell effect we're done
    callDispatcher(effStopProcess, 0, 0, NULL, 0.0);
 
@@ -1688,28 +1718,47 @@ void VSTEffect::Scan()
 #elif defined(__WXMSW__)
    TCHAR dpath[MAX_PATH];
    TCHAR tpath[MAX_PATH];
-   DWORD len = WXSIZEOF(tpath);
+   DWORD len;
 
-   // Setup the default VST path.
-   dpath[0] = '\0';
-   ExpandEnvironmentStrings(wxT("%ProgramFiles%\\Steinberg\\VSTPlugins"),
-                            dpath,
-                            WXSIZEOF(dpath));
-
-   // Check registry for the real path
+   // Try HKEY_CURRENT_USER registry key first
+   len = sizeof(tpath) / sizeof(TCHAR);
    if (SHRegGetUSValue(wxT("Software\\VST"),
-                          wxT("VSTPluginsPath"),
-                          NULL,
-                          tpath,
-                          &len,
-                          FALSE,
-                          dpath,
-                          (DWORD) _tcslen(dpath)) == ERROR_SUCCESS) {
+                       wxT("VSTPluginsPath"),
+                       NULL,
+                       tpath,
+                       &len,
+                       FALSE,
+                       NULL,
+                       0) == ERROR_SUCCESS) {
       tpath[len] = 0;
+      dpath[0] = 0;
       ExpandEnvironmentStrings(tpath, dpath, WXSIZEOF(dpath));
       wxGetApp().AddUniquePathToPathList(LAT1CTOWX(dpath), pathList);
    }
 
+   // Then try HKEY_LOCAL_MACHINE registry key
+   len = sizeof(tpath) / sizeof(TCHAR);
+   if (SHRegGetUSValue(wxT("Software\\VST"),
+                       wxT("VSTPluginsPath"),
+                       NULL,
+                       tpath,
+                       &len,
+                       TRUE,
+                       NULL,
+                       0) == ERROR_SUCCESS) {
+      tpath[len] = 0;
+      dpath[0] = 0;
+      ExpandEnvironmentStrings(tpath, dpath, WXSIZEOF(dpath));
+      wxGetApp().AddUniquePathToPathList(LAT1CTOWX(dpath), pathList);
+   }
+
+   // Add the default path last
+   dpath[0] = 0;
+   ExpandEnvironmentStrings(wxT("%ProgramFiles%\\Steinberg\\VSTPlugins"),
+                            dpath,
+                            WXSIZEOF(dpath));
+   wxGetApp().AddUniquePathToPathList(LAT1CTOWX(dpath), pathList);
+
    // Recursively scan for all DLLs
    wxGetApp().FindFilesInPathList(wxT("*.dll"), pathList, files, wxDIR_DEFAULT);
 
